/* FBContainerView.h created by ahoesch on Fri 28-Apr-2000 */

#import <AppKit/AppKit.h>

@class FBContainerView;
@class FBItemView;
@class FBObject;

@protocol FBContainerViewDataSourceProtocol

- (NSArray *)containerItems;

@end

@protocol FBContainerViewDragHandlerProtocol

- (BOOL)beginDragOperationInContainerView:(FBContainerView *)containerView;
- (void)pasteboard:(NSPasteboard *)sender provideDataForTypes:(NSArray *)types;

@end

@protocol FBContainerViewDropHandlerProtocol

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender containerView:(FBContainerView *)containerView;

@end

@interface FBContainerView : NSView
{
   id < FBContainerViewDataSourceProtocol > _dataSource;
   NSColor *_backgroundColor;
   id _delegate;
   id _target;
   SEL _doubleAction;
   id < FBContainerViewDragHandlerProtocol > _dragHandler;
   id < FBContainerViewDropHandlerProtocol > _dropHandler;
   NSArray *_acceptedPboardTypes;
   BOOL _draggingEnabled;
   BOOL _allowsEmptySelection;
   BOOL _allowsMultipleSelection;
   BOOL _allowsDeletion;
   BOOL _confirmDeletion;
}

+ (id)scrollViewWithRect:(NSRect)contentRect;

- (void)setDataSource:(id < FBContainerViewDataSourceProtocol >)dataSource;
- (void)reloadData;

- (NSArray *)selectedItemViews;
- (void)removeAllItemViews;
- (void)arrangeItemViews;
- (void)relayout;

//- (void)itemViewClicked:(NSEvent *)theEvent;
- (void)selectItemView:(FBItemView *)itemView byExtendingSelection:(BOOL)flag;
- (void)deselectItemView:(FBItemView *)itemView;
- (FBObject *)selectedItem;
- (NSArray *)selectedItems;
- (NSArray *)selectedItemViews;
- (int)numberOfSelectedItems;

- (void)setDraggingEnabled:(BOOL)flag;
- (BOOL)draggingEnabled;

- (void)setAllowsEmptySelection:(BOOL)flag;
- (BOOL)allowsEmptySelection;

- (void)setAllowsMultipleSelection:(BOOL)flag;
- (BOOL)allowsMultipleSelection;

- (void)setAllowsDeletion:(BOOL)flag;
- (BOOL)allowsDeletion;

- (void)setConfirmDeletion:(BOOL)flag;
- (BOOL)confirmDeletion;

- (void)setBackgroundColor:(NSColor *)aColor;

- (void)setDelegate:(id)delegate;
- (id)delegate;

- (void)setDoubleAction:(SEL)selector;
- (SEL)doubleAction;

- (void)setTarget:(id)target;
- (id)target;

- (void)selectItem:(FBObject *)item;
- (void)selectItemAtIndex:(int)index byExtendingSelection:(BOOL)flag;

- (void)setSelectionIndexes:(NSArray *)selectionIndexes;
- (NSArray *)selectionIndexes;

- (NSArray *)pboardTypes;

- (void)setAcceptedPboardTypes:(NSArray *)types;
- (NSArray *)acceptedPboardTypes;

- (void)setDragHandler:(id < FBContainerViewDragHandlerProtocol >)handler;
- (id < FBContainerViewDragHandlerProtocol >)dragHandler;

- (void)setDropHandler:(id < FBContainerViewDropHandlerProtocol >)handler;
- (id < FBContainerViewDropHandlerProtocol >)dropHandler;

- (void)delete:(id)sender;
- (void)clearSelection;

- (void)willDeleteItems:(NSArray *)items;

@end
