/* DatabaseController.h created by ahoesch on Fri 14-Apr-2000 */
#import <AppKit/AppKit.h>
#import <FBAccess/FBExecHandler.h>
#import <FBAccess/FBDatabaseConnection.h>
#import "FBItemView.h"

@class FBConnectionController;
@class FBDatabaseChannel;

@interface FBDatabaseController : NSObject < FBItemViewItemProtocol >
{
   NSString *_host;
   NSString *_database;
   NSString *_port;
   NSString *_options;
   FBExecHandler *_exec;
   FBDatabaseStatus _status;
   NSMutableArray *_connectionControllers;
   Class _connectionControllerClass;
}

+ (void)setConnectionControllerClassName:(NSString *)classname;

- (id)initWithHost:(NSString *)host database:(NSString *)database options:(NSString *)options createIfNotExisting:(BOOL)flag;
- (id)initWithHost:(NSString *) host port:(NSString *) database options:(NSString *)options;

- (BOOL)startDatabase;
- (BOOL)startDatabaseAndWait;
- (BOOL)stopDatabase;
- (BOOL)deleteDatabase;

- (void)databaseStopped;
- (void)shutdown;

// creates a default connection controller; class is set with +setConnectionControllerClassName:
- (FBConnectionController *)newConnectionController;

// creates a connection controller of a given subclass of FBConnectionController
- (FBConnectionController *)newConnectionControllerOfClassNamed:(NSString *)classname;

- (void)removeConnectionController:(FBConnectionController *)controller;
- (void)removeAllConnectionControllers;
- (FBConnectionController *)systemConnection;

- (void)updateStatus;

- (NSString *)host;
- (NSString *)database;
- (NSString *)port;
- (FBDatabaseStatus)status;
- (BOOL)isRunning;
- (BOOL)isStopped;

- (void)setOptions:(NSString *)aString;

- (FBExecHandler *)execHandler;
- (FBDatabaseChannel *)databaseChannel;

@end
