/* FBOutlineViewAssociation.h created by ahoesch on Fri 22-Jun-2001 */

#import <Foundation/Foundation.h>
#import "FBAssociation.h"

@protocol OutlineCellManagerProtocol

- (void)outlineView:(NSOutlineView *)outlineView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn object:(FBObject *)object;
//- (NSNumber *)outlineView:(NSOutlineView *)outlineView isObjectExpandable:(FBObject *)object;
//- (NSArray *)displayChildren:(NSArray *)children forObject:(FBObject *)object;

@end

@interface FBOutlineItem : NSObject
{
   FBObject *_object;
   NSString *_key;
   NSArray *_children;
   BOOL _editable;
}

- (id)initWithObject:(FBObject *)object key:(NSString *)key editable:(BOOL)editable;

- (FBObject *)object;
- (BOOL)editable;

- (void)setObjectValue:(id)value;
- (id)objectValue;

- (void)setChildren:(NSArray *)children;
- (NSArray *)children;

- (void)invalidateChildren;

@end

@interface FBOutlineViewAssociation : FBAssociation
{
   NSArray * _bindArray;
   NSMutableDictionary *_progressDic;
   BOOL _isDragging;
   FBOutlineItem *_rootItem;
   id < OutlineCellManagerProtocol > _outlineCellManager;
}

- (void)setBinding:(NSArray *)array;

- (NSDictionary *)progressDicForEntity:(FBEntity *)entity;
- (void)setOutlineCellManager:(id < OutlineCellManagerProtocol >)delegate;

/* ***************************************
   Drag & Drop
   *************************************** */
- (NSString *)valueRelationshipKeyForEntity:(FBEntity *)entity;
- (FBRelationship *)valueRelationshipForEntity:(FBEntity *)entity;
- (FBRelationship *)sortRelationshipForEntity:(FBEntity *)entity property:(NSString **)pString;
- (FBRelationship *)extendingRelationshipForEntity:(FBEntity *)entity;

- (void)makeRoomInObject:(FBObject *)object forNumberOfToBeInsertedItems:(int)anzahl afterItemWithSortIndex:(NSNumber *)preIndex;
- (void)refreshSortIndexesOfObject:(FBObject *)object;

- (void)insertObjects:(NSArray *)objects intoItem:(FBOutlineItem *)item atIndex:(int)index;
- (void)insertObjects:(NSArray *)objects intoObject:(FBObject *)item atIndex:(int)index;
- (void)removeObjects:(NSArray *)objects fromItem:(FBOutlineItem *)item;
- (void)removeObjects:(NSArray *)objects fromObject:(FBObject *)item;

- (FBOutlineItem *)parentForItem:(FBOutlineItem *)item;
- (FBOutlineItem *)rootItem;
- (int)indexOfChild:(id)child inItem:(id)item;

- (void)reloadItem:(FBOutlineItem *)item;
- (FBOutlineItem *)itemForObject:(FBObject *)object items:(NSArray *)items;
- (void)reloadObject:(FBObject *)object;

@end
