/* FBTableView.h created by ahoesch on Tue 20-Feb-2001 */

#import <AppKit/AppKit.h>

@class FBTableView;

@protocol FBTableViewDragHandlerProtocol

- (BOOL)beginDragOperationInTableView:(FBTableView *)tableView;

@end

@protocol FBTableViewDropHandlerProtocol

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender tableView:(FBTableView *)tableView;

@end


// This class supports dragging of objects. A subclass should override -pboardTypes and implement -pasteboard:provideDataForType: if dragging of objects is to be used. These two method can also be implemented in the delegate of the tableview (instead of subclassing). The method seDataCell:forColumnAtIndex: is provied for convenience. If you want to present both an image and text in a cell, you can use this method to set [ImageTextCell cell] or any sublclass of this class.

@interface FBTableView : NSTableView
{
   id < FBTableViewDragHandlerProtocol > _dragHandler;
   id < FBTableViewDropHandlerProtocol > _dropHandler;
   BOOL _draggingEnabled;
   BOOL _droppingEnabled;
   BOOL _allowsDeletion;
   BOOL _confirmDeletion;
   BOOL _suppressAlteredDragBehaviour;
}


- (void)setDataCell:(NSCell *)cell forColumnAtIndex:(unsigned int)index;

- (NSImage *)dragImage;

- (NSArray *)pboardTypes;		// drag
- (NSArray *)acceptedPboardTypes;	// drop

- (void)setDragHandler:(id < FBTableViewDragHandlerProtocol >)handler;
- (id < FBTableViewDragHandlerProtocol >)dragHandler;

- (void)setDropHandler:(id < FBTableViewDropHandlerProtocol >)handler;
- (id < FBTableViewDropHandlerProtocol >)dropHandler;

- (void)delete:(id)sender;

- (void)setConfirmDeletion:(BOOL)flag;
- (BOOL)confirmDeletion;

- (void)setAllowsDeletion:(BOOL)flag;
- (BOOL)allowsDeletion;

- (void)setSuppressAlteredDragBehaviour:(BOOL)flag;

@end
