/* FBConnection.h created by ahoesch on Tue 07-Nov-2000 */

#import <Foundation/Foundation.h>
#import "FBProxy.h"

@class FBConnection;

#define FBClientDidDisconnectNotification @"FBClientDidDisconnectNotification"

typedef unsigned int MessageLength;

@interface NSFileHandle (NSFileHandleMessageExtension)

- (void)sendMessage:(NSData *)message;

@end

@interface FBPort : NSObject
{
   NSFileHandle *_handle;
   NSMutableArray *_proxies;
   FBConnection *_connection;
   NSMutableData *_message;
   MessageLength _messageLength;
}

- (id)initWithFileHandle:(NSFileHandle *)handle connection:(FBConnection *)connection;
- (FBProxy *)proxyForLocal:(id)object;
- (void)removeProxy:(FBProxy *)proxy;
- (void)handleReceivedMessage:(NSData *)message;

@end

@interface FBConnection : NSObject
{
   NSFileHandle *_handle;
   NSMutableArray *_ports;
   id _rootObject;
   FBProxy *_rootProxy;
   NSMutableDictionary *_proxyDic; // Id --> Proxy
   NSCalendarDate *_lastMessageDate;
}

/* **********************************************************
   *** Server stuff
   ********************************************************** */
+ (FBConnection *)connectionWithPort:(int)port;
- (id)initWithPort:(int)aPort;
- (void)setRootObject:(id)object;
- (id)rootObject;

/* **********************************************************
   *** Client stuff
   ********************************************************** */
+ (FBConnection *)connectionWithPort:(int)port host:(NSString *)hostname;
- (id)initWithPort:(int)aPort host:(NSString *)hostname;
- (FBProxy *)rootProxy;
- (NSData *)responseForRequest:(NSData *)message;
- (void)sendCommand:(NSData *)message;

/* **********************************************************
   *** Common stuff
   ********************************************************** */
- (NSFileHandle *)fileHandle;
- (FBProxy *)proxyForLocal:(id)object;
- (FBProxy *)proxyForRemote:(id)object;
- (void)removeProxy:(FBProxy *)proxy;
- (BOOL)isServer;
- (BOOL)isClient;

- (void)setLastMessageDate:(NSCalendarDate *)date;
- (NSCalendarDate *)lastMessageDate;

@end
