/* FBDatabaseConnection.h created by ahoesch on Tue 26-Sep-2000 */

#import <Foundation/Foundation.h>
#import <FBAccess/FBDatabaseConnection.h>
#import <FBAccess/FBExecHandler.h>
#import <FBAccess/FBErrorMetaData.h>
#import <FBAccess/FBMetaData.h>

#define FBTransactionDidCommit @"FBTransactionDidCommit"
#define FBTransactionDidRollback @"FBTransactionDidRollback"
#define FBConnectFailed @"FBConnectFailed"
#define FBCreateSessionFailed @"FBCreateSessionFailed"

//typedef enum { FBShortTransaction, FBLongTransaction } FBLockingMechanism;
typedef enum { FBReadOnlyTransaction, FBOptimisticTransaction, FBPessimisticTransaction} FBTransactionMode;

// FBPessimisticTransaction: only used when absolutely necessary, e.g. for creating or altering tables
// FBOptimisticTransaction: used for storing pending changes (UPDATE, DELETE, INSERT)
// FBReadOnlyTransaction: used for fetching data

@class FBEntity;
@class FBAttribute;

NSString* FBFullUserNameForUserName(NSString *userName);

@interface FBDatabaseChannel : NSObject
{
   NSString *_database;
   NSString *_port;
   NSString *_host;
   NSString *_username;
   NSString *_password;
   NSString *_databasePassword;
   NSString *_sessionName;
   FBExecHandler *_exec;
   NSString *_fbexecpassword;
   FBDatabaseConnection *_fbdc;
   FBErrorMetaData *_errorMetaData;
   BOOL _hasTransaction;
   FBTransactionMode _mode;
}

- (id)initWithConnectionDictionary:(NSDictionary *)connectionDic;
- (id)initWithConnectionDictionary:(NSDictionary *)connectionDic fbexecPassword:(NSString *)fbexecPassword;

- (BOOL)connect;

- (BOOL)setTransactionMode:(FBTransactionMode)mode;
- (FBTransactionMode)transactionMode;

- (BOOL)setPessimistic;
- (BOOL)isPessimistic;

- (BOOL)setOptimistic;
- (BOOL)isOptimistic;

- (BOOL)setReadOnly;
- (BOOL)isReadOnly;

- (void)createSession;
- (FBDatabaseConnection *)databaseConnection;
- (BOOL)startDatabase;
- (BOOL)startDatabaseAndWait;
- (BOOL)stopDatabase;
- (BOOL)removeDatabaseFile;

- (NSString *)currentSchema;
- (NSString *)currentUser;
- (NSString *)username;

- (BOOL)executeSQL:(NSString *)sql;
- (BOOL)executeSQL:(NSString *)sql meta:(FBMetaData **)pMeta;
- (BOOL)executeSQL:(NSString *)sql meta:(FBMetaData **)pMeta versatile:(BOOL)flag;

- (BOOL)executeSQL:(NSString *)sql preFetch:(unsigned)rowCount into:(NSData**)rawData;
- (BOOL)executeSQL:(NSString *)sql preFetch:(unsigned)rowCount into:(NSData**)rawData meta:(FBMetaData **)pMeta;
- (BOOL)executeSQL:(NSString *)sql preFetch:(unsigned)rowCount into:(NSData**)rawData meta:(FBMetaData **)pMeta versatile:(BOOL)flag;

- (BOOL)commit;
- (BOOL)commitMeta:(FBMetaData **)pMeta;
- (BOOL)rollback;
- (BOOL)rollbackAndNotify:(BOOL)flag meta:(FBMetaData **)pMeta;

- (BOOL)isSystemConnection;
- (void)logString:(NSString *)aString;

- (FBErrorMetaData *)errorMetaData;

- (void)setHasTransaction:(BOOL)flag;
- (BOOL)hasTransaction;

- (FBBlobHandle *)writeBLOB:(NSData *)data;
- (NSData *)readBLOB:(FBBlobHandle *)handle;

- (FBBlobHandle *)writeCLOB:(NSString *)data;
- (NSString *)readCLOB:(FBBlobHandle *)handle;

- (NSDictionary *)convertRowToExternal:(NSDictionary *)row entity:(FBEntity *)entity;
- (NSArray *)convertRowToInternal:(NSArray *)row attributes:(NSArray *)attributes;

- (void)setSessionAuthorization:(NSString *)authorization;
- (void)setSessionAuthorization:(NSString *)authorization superPassword:(NSString *)superPassword;

- (void)setFBExecPassword:(NSString *)password;

@end
