/* FBDelayedObserver.h created by ahoesch on Wed 13-Dec-2000 */
#import <Foundation/Foundation.h>
#import "FBObserving.h"

/* ***************************************************************
   *** Ein Einfhrung in den delayed notification mechanismus ****
   ***************************************************************
- Es gibt ein Protocol FBObserving (objectWillChange:,...)
- FBDelayedObserver implementiert FBObserving
- FBDisplayGroup erbt von FBDelayedObserver
- Im Falle einer nderung in der DisplayGroup ruft diese

	[FBObserverCenter notifyObserversObjectWillChange:object mask:...];

auf. Die Maske verrt, um welche Art von nderung es sich handelt. In FBObserverCenter sind fr ein bestimmtes Objekt eine Reihe weiterer Objekte als Observer eingetragen. Diese Objekte werden benachrichtigt, da sich das bergebene Objekt sehr bald ndert 

	[observer objectWillChange:object mask:..];

Der observer registriert diese anstehende Vernderung, kann in einer berschriebenen Methode objectWillChange: evt. noch den alten Zustand erfragen und stellt dann sicher, da am Ende der event loop genau einmal subjectChanged aufgerufen wird, falls delayed eingeschaltet ist. Wenn delayed ausgeschaltet ist, wird subjectChanged stattdessen dann aufgerufen, wenn das sich verndernde Objekt [FBObserverCenter notifyObserversObjectDidChange:object mask:...] aufruft. Dadurch wird in allen observes [observer objectDidChange:object mask:..] aufgerufen.

	[observer subjectChanged];

Die Associations erben alle von FBDelayedObserver. Am Ende der event loop wird subjectChanged aufgerufen. Dadurch wissen die Associatons, wann neue Werte vond er Displaygroup zu den UI-Elementen zu populieren sind.
********************************************************************* */

   
@interface FBDelayedObserver : NSObject <FBObserving>
{
   NSMutableSet *_changedObjects;
   NSMutableDictionary *_maskDic;
   BOOL _enabled;
   BOOL _delayed;
   int _changeCount;
}

- (void)subjectChanged;
- (id)firstChangedObject;
- (NSArray *)changedObjects;

- (void)setEnabled:(BOOL)flag;
- (void)setDelayed:(BOOL)flag;
- (BOOL)delayed;

- (void)objectWillChange:(id)object mask:(NSNumber *)mask;
- (void)objectDidChange:(id)object mask:(NSNumber *)mask;

- (unsigned)notificationMaskForObject:(id)object;
- (void)_subjectChanged;

@end
