/* FBEditingContext.h created by ahoesch on Tue 26-Sep-2000 */
//#import <Foundation/Foundation.h>
#import <FBAccess/FBDatabaseConnection.h>
#import <FBAccess/FBMetaData.h>
#import "FBModel.h"
#import "FBFetchSpecification.h"
#import "FBDatabaseChannel.h"
#import "FBObject.h"
#import "FBGlobalID.h"
#import "FBUndoManager.h"

#define FBObjectsChangedInStoreNotification @"FBObjectsChangedInStoreNotification"
#define FBEditingContextDidSaveChangesNotification @"FBEditingContextDidSaveChangesNotification"
#define FBInvalidatedAllObjectsInStoreNotification @"FBInvalidatedAllObjectsInStoreNotification"

typedef enum { FBReadOnly, FBUpdateEnabled, FBDeleteEnabled } FBAccessLevel;
typedef enum { FBControlLocking, FBAccessLocking } FBLockingMode;

FOUNDATION_STATIC_INLINE BOOL FBRangesMergable(NSRange range1, NSRange range2)
{
   return ((NSIntersectionRange(range1, range2)).length > 0 ? YES : NSMaxRange(range1) == range2.location || NSMaxRange(range2) == range1.location );
}

@class FBDisplayGroup;

@interface FBEditingContext : NSObject
{
   NSMutableDictionary *_objectDic; 	// globalID --> object
   NSMutableDictionary *_insertedObjects; 	// globalID --> object
   NSMutableDictionary *_updatedObjects;  // globalID --> <keys>
   NSMutableDictionary *_upgradedObjects;  // globalID --> entityName
   NSMutableArray *_deletedObjects;

   NSMutableDictionary *_flattenedModifications;  // globalID --> { key --> { "add" --> (...), "remove --> (...) }

   FBDatabaseChannel *_connection;
   FBModel *_model;
   FBMetaData *_latestMetaData;
   NSString *_databaseIdent;

   BOOL _suppressAutoRollbackAtEndOfEventLoop;
   BOOL _autoRollbackInitiated;
   int _deleteCounter;

   FBAccessLevel _accesslevel;
   FBLockingMode _lockingMode;
   
   NSMutableArray *_deleteRelationships;
   NSMutableArray *_fetchedObjectsEntries; // { entitytName, key, range, globalIDs }   
   NSMutableArray *_editors;

   unsigned _fetchMargin;
}

+ (id)context;
+ (void)setFetchMargin:(unsigned int)margin;
+ (id)createEditingContextForModel:(FBModel *)model;
- (id)initWithModel:(FBModel *)model;

- (void)shutdown;
- (void)initiateAutoRollback;

- (void)executeDirectSQL:(NSString *)sql;
- (FBMetaData *)latestMetaData;
- (FBDatabaseConnection *)fbdc;
- (FBModel *)model;
- (BOOL)isContextEdited;
- (FBDatabaseChannel *)databaseChannel;

- (FBObject *)objectForObjectID:(FBObjectID *)objectID;
- (FBObject *)objectForGlobalID:(FBGlobalID *)globalID;
- (NSArray *)objectsForObjectIDs:(NSArray *)objectIDs;
- (NSArray *)objectsForGlobalIDs:(NSArray *)globalIDs;
- (FBObject *)objectWithPrimaryKey:(NSNumber *)primaryKey entityName:(NSString *)entityName;
- (FBObject *)objectWithPrimaryKey:(NSNumber *)primaryKey entityName:(NSString *)entityName includeInsertedObjects:(BOOL)flag;
- (NSArray *)objectsWithFetchSpecification:(FBFetchSpecification *)fetchSpecification;
- (void)fetchRowsForEntity:(FBEntity *)entity fetchSpecification:(FBFetchSpecification *)fetchSpecification result:(NSMutableArray *)result rowDic:(NSMutableDictionary *)rowDic;

- (NSArray *)alreadyFetchedObjectsForFetchSpecification:(FBFetchSpecification *)fetchSpecification;
- (void)registerObjects:(NSArray *)objects fetchSpecification:(FBFetchSpecification *)fetchSpecification range:(NSRange)range;
- (FBQualifier *)entryQualifierForFetchSpecification:(FBFetchSpecification *)fetchSpecification;

- (void)validateAttributesOfObject:(FBObject *)object;

// used to records updates to flattened toMany-relationships
- (void)addObject:(FBObject *)object toFlattenedRelationshipWithKey:(NSString *)key target:(FBObject *)target;
- (void)removeObject:(FBObject *)object fromFlattenedRelationshipWithKey:(NSString *)key target:(FBObject *)target;

// used to record objects for UPDATE
- (void)objectWillChange:(FBObject *)object property:(NSString *)key;
- (void)objectDidChange:(FBObject *)object property:(NSString *)key;
- (void)suppressAutoRollbackAtEndOfEventLoop;

- (void)setPrivileges:(unsigned int)mask forObject:(FBObject *)object;
- (void)setOwner:(NSString *)owner forObject:(FBObject *)object;
- (void)setGroup:(NSString *)group forObject:(FBObject *)object;

- (void)upgradeObject:(FBObject *)object toEntity:(FBEntity *)subEntity;
- (BOOL)insertObject:(FBObject *)object;
- (BOOL)deleteObject:(FBObject *)object;
- (NSString *)messageForDeletingObject:(FBObject *)object;

- (NSArray *)updateStatementsForChanges:(NSDictionary *)changes keys:(NSArray *)keys object:(FBObject *)object;
- (NSArray *)insertStatementsForObject:(FBObject *)object;

- (void)removeFetchedObjectsEntries;

- (void)saveChanges;
- (void)refetch;
//- (void)invalidateAllObjects;

// The following two methods can be used if only direct SQL statements have been executed (e.g. schema synchronization). If enterprise objects operations have taken place then you have to use saveChanges/refetch.
- (void)commit;
- (void)rollback;

- (void)fixPrimaryKeyCounterForEntity:(FBEntity *)entity;
- (void)generatePrimaryKeyForObject:(FBObject *)object;
- (void)collectGarbageAndInvalidate:(BOOL)flag;

- (void)replaceTemporaryGlobalID:(FBGlobalID *)tempGlobalID withGlobalID:(FBGlobalID *)globalID;

- (void)notifyObserversAboutChanges;
- (void)notifyObserversAboutSave;
- (void)notifyObserversAboutInvalidation;

- (BOOL)isInsertedObject:(FBObject *)object;
- (BOOL)isInsertedObjects:(NSArray *)objects;

- (BOOL)hasInsertedObjects;
- (BOOL)hasUpdatedObjects;
- (BOOL)hasDeletedObjects;
- (BOOL)hasChanges;

- (void)unloadObjects:(NSArray *)objects;
- (void)unloadObject:(FBObject *)object;
- (void)transactionDidEnd;

- (void)addDeleteRelationship:(FBRelationship *)relationship;
- (void)removeDeleteRelationship:(FBRelationship *)relationship;

- (void)setDeleting:(BOOL)flag;
- (BOOL)isDeleting;

- (void)setAccesslevel:(unsigned)accesslevel;
- (unsigned)accesslevel;

- (BOOL)isDeleteEnabled;
- (BOOL)isUpdateEnabled;
- (BOOL)isInsertEnabled;
- (BOOL)isReadOnly;

- (BOOL)shouldDeleteObject:(FBObject *)object;
- (BOOL)shouldDeleteObjects:(NSArray *)objects;

- (void)addEditor:(FBDisplayGroup *)editor;
- (void)removeEditor:(FBDisplayGroup *)editor;

@end
