/* FBEntity.h created by ahoesch on Wed 27-Sep-2000 */

#import <Foundation/Foundation.h>

@class FBModel;
@class FBAttribute;
@class FBRelationship;
@class FBAction;
@class FBValue;

#define USERINFOKEY_KIND @"kind"
#define USERINFOKEY_TABLE @"tableName"
#define USERINFOVALUE_VIEW @"view"

@interface FBEntity : NSObject < NSCoding >
{
   NSString *_name;
   NSString *_className;
   NSString *_externalName;
   FBModel *_model;
   NSMutableDictionary *_attributeDic;
   NSMutableDictionary *_relationshipDic;
   NSMutableDictionary *_actionDic;
   NSMutableDictionary *_valueDic;
   NSArray *_classProperties;
   NSArray *_primaryKeyAttributes;
   NSArray *_lockingAttributes;
   NSDictionary *_userInfo;
   NSMutableArray *_subEntities;
   FBEntity *_parentEntity;
   BOOL _isAbstract;
}

- (NSDictionary *)propertyList;
- (NSArray *)attributesPropertyList;
- (NSArray *)relationshipPropertyList;

- (void)setName:(NSString *)aString;
- (NSString *)name;

- (void)setClassName:(NSString *)aString;
- (NSString *)className;

- (void)setExternalName:(NSString *)aString;
- (NSString *)externalName;

- (void)setModel:(FBModel *)model;
- (FBModel *)model;

- (void)addAttribute:(FBAttribute *)attribute;
- (void)removeAttribute:(FBAttribute *)attribute;
- (NSArray *)attributes;
- (NSArray *)attributeNames;
- (NSArray *)allAttributes;
- (FBAttribute *)attributeNamed: (NSString *)attributeName;
- (FBAttribute *)attributeForDefinition: (NSString *)attributeName;
- (FBAttribute *)findAttributeNamed:(NSString *)attributeName;
- (FBAttribute *)findAttributeNamed:(NSString *)attributeName allowPaths:(BOOL)flag;
- (FBAttribute *)findAttributeNamed:(NSString *)attributeName allowPaths:(BOOL)flag searchInSubEntities:(BOOL)searchInSubEntities;

- (void)addRelationship:(FBRelationship *)relationship;
- (void)removeRelationship:(FBRelationship *)relationship;
- (NSArray *)relationships;
- (NSArray *)allRelationships;
- (NSArray *)relationshipsVisitChildren;
- (NSArray *)allRelationshipsVisitChildren;
- (NSArray *)toOneRelationships;
- (NSArray *)toManyRelationships;
- (NSArray *)allOptionalToOneClassRelationships;
- (NSArray *)allMandatoryToOneClassRelationshipsToEntitiesWithoutSubEntities;

- (FBRelationship *)relationshipNamed: (NSString *)relationshipName;
- (FBRelationship *)relationshipForDefinition:(NSString *)relationshipName;
- (FBRelationship *)findRelationshipNamed:(NSString *)relationshipName;
- (FBRelationship *)findRelationshipNamed:(NSString *)relationshipName allowPaths:(BOOL)flag;
- (FBRelationship *)findRelationshipNamed:(NSString *)relationshipName allowPaths:(BOOL)flag searchInSubEntities:(BOOL)searchInSubEntities;

- (void)addAction:(FBAction *)action;
- (void)removeAction:(FBAction *)action;
- (NSArray *)actions;
- (NSArray *)allActions;
- (FBAction *)actionNamed:(NSString *)name;
- (FBAction *)findActionNamed:(NSString *)name;

- (void)addValue:(FBValue *)value;
- (void)removeValue:(FBValue *)value;
- (NSArray *)values;
- (NSArray *)allValues;
- (FBValue *)valueNamed:(NSString *)name;
- (FBValue *)findValueNamed:(NSString *)name;

- (void)setClassProperties:(NSArray *)properties;
- (NSArray *)classProperties;
- (NSArray *)classPropertyNames;

- (void)setPrimaryKeyAttributes:(NSArray *)keys;
- (NSArray *)primaryKeyAttributes;
- (NSArray *)primaryKeyAttributeNames;
- (FBAttribute *)primaryKeyAttribute;
- (NSString *)primaryKeyAttributeName;

- (void)setAttributesUsedForLocking: (NSArray *)attributes;
- (NSArray *)attributesUsedForLocking;

- (void)addUserInfoKey:(NSString *)key value:(NSString *)value;
- (void)setUserInfo:(NSDictionary *)dictionary;
- (NSDictionary *)userInfo;

- (void)addSubEntity:(FBEntity *)child;
- (void)removeSubEntity:(FBEntity *)child;
- (NSArray *)subEntities;
- (void)setParentEntity:(FBEntity *)entity; // must not be used externally
- (FBEntity *)parentEntity;
- (FBEntity *)rootEntity;
- (NSArray *)subEntitiesVisitChildren;

- (void)setIsAbstractEntity:(BOOL)f;
- (BOOL)isAbstractEntity;
- (BOOL)isKindOfEntity:(FBEntity *)entity;
- (BOOL)isViewEntity;
- (BOOL)isValidPropertyKey:(NSString *)key;

- (BOOL)checkSortOrderings:(NSArray *)sortOrderings;

@end
