/* FBObject.h created by ahoesch on Thu 12-Oct-2000 */
#import <Foundation/Foundation.h>

@class FBEditingContext;
@class FBEntity;
@class FBRelationCarrier;
@class FBRelationship;
@class FBGlobalID;
@class FBObjectID;
@class FBQualifier;

#define FBObjectDidChangeNotification @"FBObjectDidChangeNotification"

@class FBObject;

@interface FBChangePropertyRule : NSObject
{
   NSString *_propertyName;
   NSArray *_propertyNames;
   NSArray *_isNotSetProperties;
   NSArray *_isSetProperties;
   BOOL _quick;
   BOOL _changeAllowed;
}

+ (id)ruleWithPropertyName:(NSString *)propertyName isNotSet:(NSArray *)isNotSetProperties isSet:(NSArray *)isSetProperties changeAllowed:(BOOL)changeAllowed;
+ (id)ruleWithPropertyNames:(NSArray *)propertyNames isNotSet:(NSArray *)isNotSetProperties isSet:(NSArray *)isSetProperties changeAllowed:(BOOL)changeAllowed;

- (id)initWithPropertyName:(NSString *)propertyName isNotSet:(NSArray *)isNotSetProperties isSet:(NSArray *)isSetProperties changeAllowed:(BOOL)changeAllowed;
- (id)initWithPropertyNames:(NSArray *)propertyNames isNotSet:(NSArray *)isNotSetProperties isSet:(NSArray *)isSetProperties changeAllowed:(BOOL)changeAllowed;

- (BOOL)leaveForPropertyName:(NSString *)propertyName object:(FBObject *)object message:(NSString **)pMessage;

@end

@interface FBDeleteObjectRule : NSObject
{
   NSArray *_isNotSetProperties;
   NSArray *_isSetProperties;
   BOOL _quick;
   BOOL _deleteAllowed;
}

+ (id)ruleWithIsNotSet:(NSArray *)isNotSetProperties isSet:(NSArray *)isSetProperties deleteAllowed:(BOOL)deleteAllowed;

- (id)initWithIsNotSet:(NSArray *)isNotSetProperties isSet:(NSArray *)isSetProperties deleteAllowed:(BOOL)deleteAllowed;

- (BOOL)leaveWithObject:(FBObject *)object message:(NSString **)pMessage;

@end


@interface FBObject : NSObject
{
   FBEditingContext *_editingContext;
   BOOL _isValid;
   BOOL _isValidating;
   BOOL _untouched;
   BOOL _requiresRefetchAlthoughUntouched;
   FBEntity *_entity;
   NSMutableDictionary *_valueDic;
   NSMutableDictionary *_relationDic; // key --> FBRelationCarrier
}

+ (id)createObjectForEntity:(FBEntity *)entity;
- (id)initWithEntity:(FBEntity *)entity;

- (void)setEntity:(FBEntity *)entity;
- (FBEntity *)entity;
- (FBObjectID *)objectID;
- (FBGlobalID *)globalID;

- (void)setValid;

- (void)setEditingContext:(FBEditingContext *)editingContext;
- (FBEditingContext *)editingContext;

- (FBObject *)setPrimaryKey:(NSNumber *)number;
- (NSNumber *)primaryKey;
- (NSArray *)primaryKeys;
- (BOOL)isInsertedObject;

- (NSArray *)changePropertyRules;
- (NSArray *)deleteObjectRules;

- (BOOL)shouldDelete;
- (BOOL)shouldChangeProperty:(NSString *)propertyName;
- (void)willChangeProperty:(NSString *)propertyName;
- (void)didChangeProperty:(NSString *)propertyName;

- (void)awakeFromInsertionInEditingContext:(FBEditingContext *)editingContext;
- (void)awakeFromFetchInEditingContext:(FBEditingContext *)editingContext;
- (void)awakeInEditingContext:(FBEditingContext *)editingContext;

- (void)invalidate;
- (void)invalidateValueDic;
- (void)setValues:(NSArray *)row forAttributes:(NSArray *)attributes;
- (void)takeValuesFromDic:(NSDictionary *)dic;
- (NSMutableDictionary *)valueDic;
- (NSDictionary *)keyValueDic;
- (NSDictionary *)keyValueDicForKeys:(NSArray *)keys;
- (FBRelationCarrier *)relationCarrierForKey:(NSString *)key;
- (FBRelationCarrier *)relationCarrierForKey:(NSString *)key validate:(BOOL)validate;
- (void)validateRelationCarrierForKey:(NSString *)key withValue:(id)value;
- (void)createDestinationObjectForRelationship:(FBRelationship *)relationship;

- (BOOL)setObjects:(NSArray *)objects forRelationshipWithKey:(NSString *)key removedObjects:(NSArray **)removedObjects;
- (BOOL)setObjects:(NSArray *)objects forRelationshipWithKey:(NSString *)key;
- (BOOL)addObjects:(NSArray *)objects toBothSidesOfRelationshipWithKey:(NSString *)key;
- (BOOL)addObject:(FBObject *)object toBothSidesOfRelationshipWithKey:(NSString *)key;
- (BOOL)removeObject:(FBObject *)object fromBothSidesOfRelationshipWithKey:(NSString *)key;
- (BOOL)removeObject:(FBObject *)object fromBothSidesOfRelationshipWithKey:(NSString *)key preventDeletion:(BOOL)flag;
- (BOOL)removeObjects:(NSArray *)objects fromBothSidesOfRelationshipWithKey:(NSString *)key preventDeletion:(BOOL)flag;
- (BOOL)removeObjects:(NSArray *)objects fromBothSidesOfRelationshipWithKey:(NSString *)key;
- (BOOL)removeAllObjectsFromBothSidesOfRelationshipWithKey:(NSString *)key preventDeletion:(BOOL)flag;

- (void)takeValue:(id)value forKey:(NSString *)key;
- (id)valueForKey:(NSString *)key;

- (NSArray *)valuesForDetailKey:(NSString *)key additionalQualifier:(FBQualifier *)additionalQualifier;

- (void)become:(NSString *)entityName;

- (void)setRLPPrivileges:(unsigned int)mask;
- (void)setRLPOwner:(NSString *)owner;
- (void)setRLPGroup:(NSString *)group;

- (NSException *)validateForSave;
- (NSException *)validateForDelete;
- (void)transactionDidEnd;

@end
