/* FBSQLExpression.h created by ahoesch on Tue 10-Oct-2000 */
#import <Foundation/Foundation.h>
#import "FBRelationship.h"
#import "FBQualifier.h"

@class FBFetchSpecification;
@class FBKeyValueQualifier;
@class FBKeyComparisonQualifier;
@class FBModel;
@class FBEntity;
@class FBAttribute;
@class FBDatabaseChannel;
@class FBObject;

@interface FBSQLExpression : NSObject
{
    FBEntity *_entity;
    NSMutableString *_listString;  // FIRSTNAME,LASTANAE,...
    NSMutableString *_valueString;   // 1234,'Andreas',...
    NSMutableString *_orderByString; // LASTNAME desc
    NSMutableString *_joinClauseString;  // t0.departmentId = t1.departmentId and ...
    NSString *_whereClauseString;    // CITY = 'Luebeck'
    NSString *_statement;
    NSMutableArray *_aliasesByTableNameDictionaries; // ( tableName -> alias, ...)
    NSMutableArray *_aliasesByRelationshipDictionaries; // ( @"person", --> @"t2", @"person.address" --> @"t3")
    NSMutableArray *_inheritanceEntities; // ( ( Business) )
    NSMutableArray *_bindings;
    NSMutableArray *_contextStack;
    NSMutableArray *_entityAttributesArray;
    int _useAliases;
    int _aliasCounter;
    int _branchIndex;
}

// NEW_APPROACH
+ (FBSQLExpression *)selectStatementForFetchSpecification:(FBFetchSpecification *)fetchSpecification entity:(FBEntity *)entity;
// NEW_APPROACH
- (void)prepareSelectExpressionWithFetchSpecification:(FBFetchSpecification *)fetchSpecification;

+ (FBSQLExpression *)expressionForString:(NSString *)string;

/* ***********************************************
   data access
   *********************************************** */
+ (FBSQLExpression *)selectStatementForAttributes:(NSArray *)attributes lock:(BOOL)flag fetchSpecification:(FBFetchSpecification *)fetchSpecification entity:(FBEntity *)entity;
+ (FBSQLExpression *)fixStatementForPrimaryKeyCounter:(FBEntity *)entity;
+ (FBSQLExpression *)selectStatementForPrimaryKey:(FBEntity *)entity;
+ (FBSQLExpression *)selectStatementForPath:(NSArray *)path key:(int)value;

+ (FBSQLExpression *)updateStatementForRow:(NSDictionary *)row qualifier:(FBQualifier *)qualifier entity:(FBEntity *)entity;

+ (FBSQLExpression *)insertStatementForRow:(NSDictionary *)row entity:(FBEntity *)entity;

+ (FBSQLExpression *)privilegesStatementWithQualifier:(FBQualifier *)qualifier entity:(FBEntity *)entity mask:(unsigned int)mask;

+ (FBSQLExpression *)setOwnerStatementWithQualifier:(FBQualifier *)qualifier entity:(FBEntity *)entity owner:(NSString *)owner;

+ (FBSQLExpression *)setGroupStatementWithQualifier:(FBQualifier *)qualifier entity:(FBEntity *)entity group:(NSString *)group;

+ (FBSQLExpression *)deleteStatementWithQualifier:(FBQualifier *)qualifier entity:(FBEntity *)entity;
+ (NSArray *)deleteStatementsWithQualifier:(FBQualifier *)qualifier entity:(FBEntity *)entity;
+ (NSArray *)deleteStatementsForObject:(FBObject *)object selectStatements:(NSMutableDictionary *)selectStatements recursions:(unsigned int)recursions expression:(FBSQLExpression **)pExpression;
+ (NSArray *)deleteStatementsForObject:(FBObject *)object path:(NSArray *)path selectStatements:(NSMutableDictionary *)selectStatements recursions:(unsigned int)numberOfToleratedRecursions expression:(FBSQLExpression **)pExpression;
+ (NSArray *)deleteStatementsForPath:(NSArray *)path key:(int)value;
+ (NSArray *)deleteStatementsForPath:(NSArray *)path anchor:(FBEntity *)anchor entity:(FBEntity *)entity key:(int)value;

/* ***********************************************
   schema creation
   *********************************************** */
+ (NSArray *)createSchemaStatementsForSchema:(NSString *)name authorization:(NSString *)user;
+ (NSArray *)dropSchemaStatementsForSchema:(NSString *)name;
+ (NSArray *)createTableStatementsForEntity:(FBEntity *)entity;
+ (NSArray *)createViewStatementsForEntity:(FBEntity *)entity;
+ (NSArray *)statementsToInsertColumnForAttribute:(FBAttribute *)attribute;

- (void)addCreateClauseForAttribute:(FBAttribute *)attribute;
+ (NSString *)columnTypeStringForAttribute:(FBAttribute *)attribute;
- (NSString *)allowsNullClauseForConstraint:(BOOL)flag;

+ (NSArray *)dropViewStatementsForViewNamed:(NSString *)viewName;
+ (NSArray *)dropTableStatementsForTableNamed:(NSString *)tableName;
+ (NSArray *)statementsToDeleteColumnNamed:(NSString *)columnName inTableNamed:(NSString *)tableName;

+ (NSArray *)statementsToDropConstraint:(NSString *)constraintName fromTable:(NSString *)tableName;
+ (NSArray *)primaryKeyConstraintStatementsForEntity:(FBEntity *)entity;
+ (NSArray *)foreignKeyConstraintStatementsForRelationship:(FBRelationship *)relationship;

/* ***********************************************
   schema synchronization
   *********************************************** */
+ (NSArray *)statementsToUpdateSchema:(NSString *)schema forModel:(FBModel *)model databaseChannel:(FBDatabaseChannel *)databaseChannel;
+ (NSArray *)statementsToUpdateSchema:(NSString *)schema forModel:(FBModel *)model databaseChannel:(FBDatabaseChannel *)databaseChannel allowDestruction:(BOOL)flag;
+ (NSArray *)statementsToUpdateConstraints:(NSString *)schema forModel:(FBModel *)model databaseChannel:(FBDatabaseChannel *)databaseChannel;

/* returns the name of a primary key constraint that is no longer referenced by entity. This method is used to determine wich primary key constraint has to be removed */
+ (NSString *)invalidPrimaryKeyConstraint:(NSDictionary *)primaryKey entity:(FBEntity *)entity;

/* returns the names of all foreign key constraints that are no longer referenced by relationships. This method is used to determine which foreign key constraints have to be removed */
+ (NSArray *)invalidForeignKeyConstraints:(NSArray *)foreignKeys entity:(FBEntity *)entity;

/* returns the name of the primary key constraint for entity */
+ (NSString *)primaryKeyConstraint:(NSDictionary *)primaryKey entity:(FBEntity *)entity;

/* returns the name of the foreign key constraint that uses all passed attributes */
+ (NSString *)foreignKeyConstraint:(NSArray *)foreignKeys relationship:(FBRelationship *)relationship;

+ (NSString *)notNullConstraint:(NSArray *)checks column:(NSString *)columnName;

+ (NSArray *)statementsToConvertTypeOfAttribute:(FBAttribute *)attribute;

- (id)initWithEntity:(FBEntity *)entity;

- (void)prepareSelectExpressionWithAttributes:(NSArray *)attributes lock:(BOOL)lock fetchSpecification:(FBFetchSpecification *)fetchSpecification;

- (void)prepareUpdateExpressionWithRow:(NSDictionary *)row qualifier:(FBQualifier *)qualifier;

- (void)prepareInsertExpressionWithRow:(NSDictionary *)row;

- (void)prepareDeleteExpressionForQualifier:(FBQualifier *)qualifier;

- (void)preparePrivilegesExpressionForQualifier:(FBQualifier *)qualifier mask:(unsigned int)mask;
- (void)prepareSetOwnerExpressionForQualifier:(FBQualifier *)qualifier owner:(NSString *)owner;
- (void)prepareSetGroupExpressionForQualifier:(FBQualifier *)qualifier group:(NSString *) group;

- (void)prepareDeleteExpressionForPath:(NSArray *)path key:(int)value;
- (void)prepareSelectExpressionForPath:(NSArray *)path key:(int)value;

- (NSString *)assembleSelectStatementWithAttributes:(NSArray *)attributes lock:(BOOL)lock qualifier:(FBQualifier *)qualifier fetchOrder:(NSArray *)fetchOrder selectString:(NSString *)selectString columnList:(NSString *)columnList tableList:(NSString *)tableList whereClause:(NSString *)whereClause joinClause:(NSString *)joinClause orderByClause:(NSString *)orderByClause lockClause:(NSString *)lockClause;
- (NSString *)assembleSelectStatementForEntity:(FBEntity *)entity tableList:(NSString *)tableList whereClause:(NSString *)whereClause;

- (NSString *)assembleUpdateStatementWithRow:(NSDictionary *)row qualifier:(FBQualifier *)qualifier tableList:(NSString *)tableList updateList:(NSString *)updateList whereClause:(NSString *)whereClause;

- (NSString *)assembleInsertStatementWithRow:(NSDictionary *)row tableList:(NSString *)tableList columnList:(NSString *)columnList valueList:(NSString *)valueList;

- (NSString *) assembleDeleteStatementTableList:(NSString *)tableList whereClause:(NSString *)whereClause;

- (void)addSelectListAttribute:(FBAttribute *)attribute;
- (void)addUpdateListAttribute:(FBAttribute *)attribute value:(NSString *)value;
- (void)addInsertListAttribute:(FBAttribute *)attribute value:(NSString *)value;

- (void)appendItem:(NSString *)itemString toListString:(NSMutableString *)listString;

- (void)setStatement:(NSString *)statement;
- (NSString *)statement;
- (FBEntity *)entity;

- (NSString *)sqlStringForAttributeNamed:(NSString *)name;
- (NSString *)sqlStringForAttribute:(FBAttribute *)attribute;
- (NSString *)sqlStringForAttributePath:(NSArray *)path;
- (NSString *)sqlStringForSelector:(SEL)selector value:(id)value;
- (NSString *)sqlStringForValue:(id)value attributeNamed:(NSString *)name;

- (NSString *)sqlStringForSQLQualifier:(FBSQLQualifier *)qualifier;
- (NSString *)sqlStringForKeyValueQualifier:(FBKeyValueQualifier *)qualifier;
- (NSString *)sqlStringForKeyArrayQualifier:(FBKeyArrayQualifier *)qualifier;
- (NSString *)sqlStringForInQualifier:(FBInQualifier *)qualifier;
- (NSString *)sqlStringForNotInQualifier:(FBInQualifier *)qualifier;
- (NSString *)sqlStringForKeyComparisonQualifier:(FBKeyComparisonQualifier *)qualifier;
- (NSString *)sqlStringForKeyIsNullQualifier:(FBKeyIsNullQualifier *)qualifier;
- (NSString *)sqlStringForKeyIsNotNullQualifier:(FBKeyIsNotNullQualifier *)qualifier;
- (NSString *)sqlStringForIntermediateQualifier:(FBIntermediateQualifier *)qualifier;
- (NSString *)sqlStringForConjoinedQualifiers:(NSArray *)qualifiers;
- (NSString *)sqlStringForDisjoinedQualifiers:(NSArray *)qualifiers;

+ (NSString *)formatSQLString:(NSString *)sqlString format:(NSString *)format;
+ (NSString *)formatValue:(id)value forAttribute:(FBAttribute *)attribute;
+ (NSString *)formatBit:(NSData *)data;
+ (NSString *)formatStringValue:(NSString *)string;

- (void)joinExpression;
- (void)addJoinClauseWithLeftName:(NSString *)leftName rightName:(NSString *)rightName joinSemantic:(FBJoinSemantic)semantic;
- (NSString *)assembleJoinClauseWithLeftName:(NSString *)leftName rightName:(NSString *)rightName joinSemantic:(FBJoinSemantic)semantic;

- (NSString *)tableListWithRootEntity:(FBEntity *)entity;
- (NSString *)tableListWithRootEntity:(FBEntity *)entity allowJoins:(BOOL)flag;
//- (NSString *)naturalJoinStringForEntity:(FBEntity *)entity;

- (void)setUseAliases:(BOOL)flag;
- (BOOL)useAliases;

- (NSMutableDictionary *)aliasesByTableNameDic;
- (NSMutableDictionary *)aliasesByRelationshipPath;
- (NSMutableSet *)inheritanceSet;

- (NSString *)whereClauseString;
- (NSString *)joinClauseString;

- (NSString *)listString;

- (void)registerEntity:(FBEntity *)entity;
- (void)registerTableName:(NSString *)tableName;
- (void)registerKeyPath:(NSString *)keyPath;

- (NSString *)aliasForEntity:(FBEntity *)entity atIndex:(int)index;
- (NSString *)aliasForTableName:(NSString *)tableName atIndex:(int)index;

- (NSString *)aliasForEntity:(FBEntity *)entity;
- (NSString *)aliasForTableName:(NSString *)tableName;

- (void)newBranch;
- (int)currentBranchIndex;
- (NSDictionary *)aliasToTableNameDic;
- (int)indexForRelationshipPath:(NSString *)path;
- (void)setBranchIndex:(int)index;
- (NSString *)joinClauseForBranchIndex:(int)branchIndex;
- (BOOL)shouldCreateNewBranchForAttributePath:(NSArray *)path;

- (NSArray *)entityAttributesArray;
- (unsigned)startIndexForNextAttributes;
- (void)addSelectListAttributesForEntity:(FBEntity *)entity keyPath:(NSString *)keyPath ownerIndex:(int)ownerIndex detailKey:(NSString *)detailKey;

- (NSArray *)relatedEntitiesForEntity:(FBEntity *)currentEntity nextRelationship:(FBRelationship *)nextRelationship;

@end


@interface NSString (FBSQLFormatting)

- (NSString *)stringAfterRemovingAllOccurancesOf:(NSString *)string;
+ (NSString *)tableNameForExternalName:(NSString *)table schema:(NSString *)schema;
+ (NSString *)viewNameForExternalName:(NSString *)table schema:(NSString *)schema;
+ (NSString *)fullyQualifiedNameForTable:(NSString *)table schema:(NSString *)schema;
+ (NSString *)fullyQualifiedNameForView:(NSString *)view schema:(NSString *)schema;
+ (NSString *)fullyQualifiedNameForColumn:(NSString *)column table:(NSString *)table schema:(NSString *)schema;

- (NSString *)sqlString;

+ (NSString *)whereClauseForPath:(NSArray *)path key:(int)value bracketCounter:(int *)pCount;

@end

@interface NSNumber (FBSQLFormatting)

- (NSString *)sqlString;

@end

@interface FBEntity (SQLExpression)

- (BOOL)isPrimaryKeyAttributes:(NSArray *)attributes;
- (BOOL)externalNameHasSchema;

@end

