/* OBModelConverter.h created by ahoesch on Fri 01-Dec-2000 */
#import <Foundation/Foundation.h>
#import <FBEnterprise/FBEnterprise.h>

@class OBModel;
@class OBClass;
@class OBAttribute;

@interface OBModelConverter : NSObject
{
   OBModel *_obmodel;
   FBModel *_fbmodel;
}

+ (FBModel *)fbmodelForOBClass:(OBClass *)aClass;
+ (FBModel *)fbmodelForOBModel:(OBModel *)obmodel;

- (id)initWithOBModel:(OBModel *)model;

- (OBModel *)obmodel;
- (FBModel *)fbmodelIncludingMeta:(BOOL)flag;
//- (FBModel *)fbmodelWithTableNamePrefix:(NSString *)prefix;

- (void)convert:(BOOL)flag;

- (void)addExtraForClass:(OBClass *)aClass;
- (void)addClass:(OBClass *)aClass;
- (void)addSubClass:(OBClass *)aClass;
- (void)addRole:(OBClass *)aClass;
- (void)addAttribute:(OBAttribute *)attribute toClass:(OBClass *)aClass;

/*- (NSString *)entityNameForClass:(OBClass *)aClass attribute:(OBAttribute )attribute;
- (NSString *)externalNameForClass:(OBClass *)aClass;
- (NSString *)classNameForClass:(OBClass *)aClass;
- (NSString *)primaryKeyNameForClass:(OBClass *)aClass;*/

+ (NSString *)columnNameForAttribute:(FBAttribute *)attribute;
+ (NSString *)toDestRelationshipNameForToOneRoleWithName:(NSString *)rolename;
+ (NSString *)toDestRelationshipNameForToManyRoleWithName:(NSString *)rolename;
+ (NSString *)toSourceRelationshipNameForRole:(OBClass *)aClass;
+ (NSString *)keyType;
+ (NSString *)timeType;

@end

@interface FBModel (Converter)

- (NSArray *)nonMetaAndNonViewEntities;
- (void)makeAllEntitiesMeta;

@end

@interface FBEntity (Converter)

- (BOOL)isMetaEntity;

- (FBEntity *)initWithClass:(OBClass *)aClass;
- (FBEntity *)initWithName:(NSString *)name;
- (FBEntity *)initWithName:(NSString *)name createPrimaryKey:(BOOL)flag;
- (FBAttribute *)addForeignKeyForEntity:(FBEntity *)entity;
- (FBAttribute *)addForeignForAttribute:(OBAttribute *)attribute;
- (FBAttribute *)addForeignKeyWithName:(NSString *)name;
- (FBAttribute *)addAttributeWithName:(NSString *)name type:(NSString *)type;
- (FBAttribute *)addAttributeWithName:(NSString *)name type:(NSString *)type allowsNull:(BOOL)flag width:(unsigned)width;

@end

@interface FBEditingContext (Converter)

- (NSString *)databaseIdent;

@end

@interface FBAttribute (Converter)

- (void)setTypeInfo:(NSDictionary *)info;

@end

@interface OBModelConverter (Modeling)

- (void)_addAttribute:(OBAttribute *)attribute toClass:(OBClass *)aClass;
- (void)_addAttributes:(OBAttribute *)attribute toClass:(OBClass *)aClass;

- (void)_addOneToManyWithBack:(OBAttribute *)attribute toClass:(OBClass *)class deleteRule:(FBDeleteRule)deleteRule ownsDestination:(BOOL)ownsDestination;

//- (void)_addAttributesWithBack:(OBAttribute *)attribute toClass:(OBClass *)aClass;
- (void)_addReference:(OBAttribute *)attribute toClass:(OBClass *)aClass;
- (void)_addReferences:(OBAttribute *)attribute toClass:(OBClass *)aClass;
//- (void)_addReferencesWithBack:(OBAttribute *)attribute toClass:(OBClass *)aClass;
- (void)_addManyToMany:(OBAttribute *)attribute toClass:(OBClass *)aClass;
- (void)_addHistory:(OBAttribute *)attribute toClass:(OBClass *)aClass;

- (void)_addActionsForAttribute:(OBAttribute *)attribute entity:(FBEntity *)entity;

+ (NSString *)extraEntityNameForEntityNamed:(NSString *)entityName;
+ (NSString *)helpEntityNameForAttributeNamed:(NSString *)attributeName ofClassNamed:(NSString *)className;
+ (NSString *)relationshipNameForHelpEntityNamed:(NSString *)name;
+ (NSString *)relationshipNameForHistoryAttributeNamed:(NSString *)name;
+ (NSString *)relationshipNameForToManyAttributeNamed:(NSString *)name;
+ (NSString *)backRelationshipNameForRelationshipWithName:(NSString *)relName sourceEntity:(NSString *)entityName;
+ (NSString *)propertyNameForValueInIntermediate;

@end

