/* ObjectArchiver.h created by ahoesch on Fri 18-May-2001 */
#import <AppKit/AppKit.h>
#import <SAFoundation/SAFoundation.h>

#define FBArchivedObjectsPboardType	@"FBArchivedObjectsPboardType"

//@class Record;
#import "Record.h"


// An archive can contain all kinds of database objects (Books, Sections, Attachments,...). In order for the opener of an archive to know what objects are to be considered as root objects, the archive maintains rootIndexes. The variable _text allows to archive an arbitrary substring of a section. If this variable is set, all rootIndexes should be empty, since _text is then the only root object.
@interface ObjectArchiver : NSObject
{
   NSDictionary *_textPropDic;			// used for text fragments
   
   NSMutableDictionary *_bookIndexDic;		// bookId --> bookIndex
   NSMutableDictionary *_bookPropDic; 		// bookIndex --> propertyList
   NSMutableDictionary *_bookRecordDic; 	// bookIndex --> records
   NSMutableArray *_bookRootIndexes;

   NSMutableDictionary *_chapterIndexDic;	// chapterId --> chapterIndex
   NSMutableDictionary *_chapterPropDic; 	// chapterIndex --> propertyList
   NSMutableDictionary *_chapterRecordDic; 	// chapterIndex --> records

   NSMutableDictionary *_folderIndexDic;	// folderId --> folderIndex
   NSMutableDictionary *_folderPropDic; 	// folderIndex --> propertyList
   NSMutableDictionary *_folderRecordDic; 	// folderIndex --> records
   NSMutableArray *_folderRootIndexes;

   NSMutableDictionary *_fileIndexDic;		// fileId --> fileIndex
   NSMutableDictionary *_filePropDic; 		// fileIndex --> propertyList
   NSMutableDictionary *_fileRecordDic; 	// fileIndex --> records
   
   NSMutableDictionary *_sectionIndexDic;	// sectionId --> sectionIndex
   NSMutableDictionary *_sectionPropDic; 	// sectionIndex --> propertyList
   NSMutableDictionary *_sectionRecordDic; 	// sectionIndex --> records
   NSMutableArray *_sectionRootIndexes;

   NSMutableDictionary *_categoryIndexDic;	// categoryId --> categoryIndex
   NSMutableDictionary *_categoryPropDic; 	// categoryIndex --> propertyList
   NSMutableDictionary *_categoryRecordDic; 	// categoryIndex --> records

   NSMutableDictionary *_bodyIndexDic;		// bodyId --> bodyIndex
   NSMutableDictionary *_bodyPropDic; 		// bodyIndex --> propertyList
   NSMutableDictionary *_bodyRecordDic; 	// bodyIndex --> records

   NSMutableDictionary *_propertyIndexDic;
   NSMutableDictionary *_propertyPropDic;
   NSMutableDictionary *_propertyRecordDic;
   NSMutableArray *_propertyRootIndexes;

   NSMutableDictionary *_linkIndexDic;
   NSMutableDictionary *_linkPropDic;
   NSMutableDictionary *_linkRecordDic;

   NSMutableDictionary *_bibliographyIndexDic;
   NSMutableDictionary *_bibliographyPropDic;
   NSMutableDictionary *_bibliographyRecordDic;
   NSMutableArray *_bibliographyRootIndexes;
}

+ (NSData *)archivedObjects:(NSArray *)objects;
+ (NSData *)archivedRange:(NSRange)range inAttributedString:(NSAttributedString *)attrString;

- (NSDictionary *)archivedObjects:(NSArray *)objects;
- (NSDictionary *)archivedRange:(NSRange)range inAttributedString:(NSAttributedString *)attrString;

- (void)setPropertyListForTextFragment:(NSDictionary *)dic;
- (void)registerSections:(NSArray *)section;
- (NSNumber *)indexForBody:(Body *)body;

- (BOOL)bookAlreadyEncoded:(Book *)book;
- (void)setPropertyList:(NSDictionary *)dic forBook:(Book *)book;
- (NSNumber *)indexForBook:(FBObject *)book;
- (NSArray *)indexesForBooks:(NSArray *)books;
- (void)addBookRootId:(NSNumber *)number;

- (BOOL)folderAlreadyEncoded:(Folder *)folder;
- (void)setPropertyList:(NSDictionary *)dic forFolder:(Folder *)folder;
- (NSNumber *)indexForFolder:(FBObject *)folder;
- (NSArray *)indexesForFolders:(NSArray *)folders;
- (void)addFolderRootId:(NSNumber *)number;

- (BOOL)chapterAlreadyEncoded:(Chapter *)chapter;
- (void)setPropertyList:(NSDictionary *)dic forChapter:(Chapter *)chapter;
- (NSNumber *)indexForChapter:(FBObject *)chapter;
- (NSArray *)indexesForChapters:(NSArray *)chapters;

- (BOOL)fileAlreadyEncoded:(FolderFile *)file;
- (void)setPropertyList:(NSDictionary *)dic forFile:(FolderFile *)file;
- (NSNumber *)indexForFile:(FBObject *)file;
- (NSArray *)indexesForFiles:(NSArray *)files;

- (BOOL)sectionAlreadyEncoded:(Section *)section;
- (void)setPropertyList:(NSDictionary *)dic forSection:(Section *)section;
- (NSNumber *)indexForSection:(FBObject *)section;
- (NSArray *)indexesForSections:(NSArray *)sections;
- (void)addSectionRootId:(NSNumber *)number;

- (BOOL)categoryAlreadyEncoded:(Category *)category;
- (void)setPropertyList:(NSDictionary *)dic forCategory:(Category *)category;
- (NSNumber *)indexForCategory:(FBObject *)category;
- (NSArray *)indexesForCategories:(NSArray *)categories;

- (BOOL)bodyAlreadyEncoded:(Body *)body;
- (void)setPropertyList:(NSDictionary *)dic forBody:(Body *)body;
- (NSArray *)indexesForBodies:(NSArray *)bodys;

- (BOOL)propertyAlreadyEncoded:(Property *)property;
- (void)setPropertyList:(NSDictionary *)dic forProperty:(Property *)property;
- (NSNumber *)indexForProperty:(FBObject *)property;
- (NSArray *)indexesForProperties:(NSArray *)propertys;
- (void)addPropertyRootId:(NSNumber *)number;

- (BOOL)linkAlreadyEncoded:(Link *)link;
- (void)setPropertyList:(NSDictionary *)dic forLink:(Link *)link;
- (NSNumber *)indexForLink:(Link *)link;
- (NSArray *)indexesForLinks:(NSArray *)links;

- (BOOL)bibliographyAlreadyEncoded:(Bibliography *)bibliography;
- (void)setPropertyList:(NSDictionary *)dic forBibliography:(Bibliography *)bibliography;
- (NSNumber *)indexForBibliography:(FBObject *)bibliography;
- (NSArray *)indexesForBibliographies:(NSArray *)bibliographys;
- (void)addBibliographyRootId:(NSNumber *)number;

@end


@interface ObjectUnarchiver : NSObject
{   
   NSMutableDictionary *_bookDic;		// bookIndex --> Section
   NSMutableDictionary *_bookPropDic; 		// bookIndex --> propertyList
   NSMutableDictionary *_bookRecordDic; 	// bookIndex --> records

   NSMutableDictionary *_folderDic;		// folderIndex --> Section
   NSMutableDictionary *_folderPropDic; 		// folderIndex --> propertyList
   NSMutableDictionary *_folderRecordDic; 	// folderIndex --> records

   NSMutableDictionary *_chapterDic;		// chapterIndex --> Section
   NSMutableDictionary *_chapterPropDic; 		// chapterIndex --> propertyList
   NSMutableDictionary *_chapterRecordDic; 	// chapterIndex --> records

   NSMutableDictionary *_fileDic;		// fileIndex --> Section
   NSMutableDictionary *_filePropDic; 		// fileIndex --> propertyList
   NSMutableDictionary *_fileRecordDic; 	// fileIndex --> records

   NSMutableDictionary *_sectionDic;		// sectionIndex --> Section
   NSMutableDictionary *_sectionPropDic; 	// sectionIndex --> propertyList
   NSMutableDictionary *_sectionRecordDic; 	// sectionIndex --> records

   NSMutableDictionary *_categoryDic;		// categoryIndex --> Section
   NSMutableDictionary *_categoryPropDic; 	// categoryIndex --> propertyList
   NSMutableDictionary *_categoryRecordDic; 	// categoryIndex --> records

   NSMutableDictionary *_bodyDic;		// bodyIndex --> Body
   NSMutableDictionary *_bodyPropDic; 		// bodyIndex --> propertyList
   NSMutableDictionary *_bodyRecordDic; 	// bodyIndex --> records

   NSMutableDictionary *_propertyDic;		// propertyIndex --> Property
   NSMutableDictionary *_propertyPropDic;	// propertyIndex --> propertyList
   NSMutableDictionary *_propertyRecordDic;

   NSMutableDictionary *_linkDic;
   NSMutableDictionary *_linkPropDic;
   NSMutableDictionary *_linkRecordDic;

   NSMutableDictionary *_bibliographyDic;
   NSMutableDictionary *_bibliographyPropDic;
   NSMutableDictionary *_bibliographyRecordDic;

   NSMutableArray *_superflousSections;
   NSMutableArray *_neededSections;

   BOOL _ignoreRecords;
   NSDictionary *_textPropDic;

   // the following dictionaries are needed to import old format packages
   NSMutableDictionary *_emphasisDic; // propertyIndex --> {class = Emphasis; kind = 0; range = "{82, 6}"; }
   NSMutableDictionary *_attachmentDic;
}

- (NSMutableDictionary *)emphasisDic;
- (NSMutableDictionary *)attachmentDic;
- (NSMutableDictionary *)propertyPropDic;
- (NSMutableDictionary *)linkPropDic;

- (BOOL)shouldCreateNewPropertyForPropertyIndex:(NSNumber *)propertyIndex;
- (BOOL)shouldCreateNewLinkForLinkIndex:(NSNumber *)linkIndex;
- (BOOL)shouldCreateNewBodyForBodyIndex:(NSNumber *)bodyIndex;
- (BOOL)shouldCreateNewSectionForSectionIndex:(NSNumber *)sectionIndex;

- (void)addSuperflousSections:(NSArray *)array;
- (void)addNeededSection:(FBObject *)object;
- (void)updateRecordsForObject:(FBObject *)object records:(NSArray *)array;

//+ (NSArray *)unarchivedObjects:(NSData *)data editingContext:(FBEditingContext *)editingContext;
+ (NSArray *)unarchivedObjects:(NSData *)data editingContext:(FBEditingContext *)editingContext ignoreRecords:(BOOL)flag;
+ (NSMutableAttributedString *)attributedStringFromArchive:(NSData *)data body:(Body *)body editingContext:(FBEditingContext *)editingContext subsections:(NSArray **)pSubsections countCenter:(CountCenter *)countCenter;

- (NSArray *)unarchivedObjects:(NSDictionary *)dic editingContext:(FBEditingContext *)editingContext;
- (NSArray *)unarchivedObjects:(NSDictionary *)dic editingContext:(FBEditingContext *)editingContext ignoreRecords:(BOOL)flag;
- (NSMutableAttributedString *)attributedStringFromArchive:(NSDictionary *)dic body:(Body *)body editingContext:(FBEditingContext *)editingContext subsections:(NSArray **)pSubsections countCenter:(CountCenter *)countCenter;


- (void)setBook:(Book *)book forBookIndex:(NSNumber *)bookIndex;
- (Book *)bookForBookIndex:(NSNumber *)bookIndex;
- (NSDictionary *)bookPropForBookIndex:(NSNumber *)bookIndex;
- (NSArray *)bookRecordsForBookIndex:(NSNumber *)bookIndex;
- (NSNumber *)bookIdForBookIndex:(NSNumber *)bookIndex editingContext:(FBEditingContext *)editingContext;

- (void)setFolder:(Folder *)folder forFolderIndex:(NSNumber *)folderIndex;
- (Folder *)folderForFolderIndex:(NSNumber *)folderIndex;
- (NSDictionary *)folderPropForFolderIndex:(NSNumber *)folderIndex;
- (NSArray *)folderRecordsForFolderIndex:(NSNumber *)folderIndex;
- (NSNumber *)folderIdForFolderIndex:(NSNumber *)folderIndex editingContext:(FBEditingContext *)editingContext;

- (void)setChapter:(Chapter *)chapter forChapterIndex:(NSNumber *)chapterIndex;
- (Chapter *)chapterForChapterIndex:(NSNumber *)chapterIndex;
- (NSDictionary *)chapterPropForChapterIndex:(NSNumber *)chapterIndex;
- (NSArray *)chapterRecordsForChapterIndex:(NSNumber *)chapterIndex;
- (NSNumber *)chapterIdForChapterIndex:(NSNumber *)chapterIndex editingContext:(FBEditingContext *)editingContext;

- (void)setFile:(FolderFile *)file forFileIndex:(NSNumber *)fileIndex;
- (FolderFile *)fileForFileIndex:(NSNumber *)fileIndex;
- (NSDictionary *)filePropForFileIndex:(NSNumber *)fileIndex;
- (NSArray *)fileRecordsForFileIndex:(NSNumber *)fileIndex;
- (NSNumber *)fileIdForFileIndex:(NSNumber *)fileIndex editingContext:(FBEditingContext *)editingContext;

- (void)setSection:(Section *)section forSectionIndex:(NSNumber *)sectionIndex;
- (Section *)sectionForSectionIndex:(NSNumber *)sectionIndex;
- (NSDictionary *)sectionPropForSectionIndex:(NSNumber *)sectionIndex;
- (NSArray *)sectionRecordsForSectionIndex:(NSNumber *)sectionIndex;
- (NSNumber *)sectionIdForSectionIndex:(NSNumber *)sectionIndex editingContext:(FBEditingContext *)editingContext;

- (void)setCategory:(Category *)category forCategoryIndex:(NSNumber *)categoryIndex;
- (Category *)categoryForCategoryIndex:(NSNumber *)categoryIndex;
- (NSDictionary *)categoryPropForCategoryIndex:(NSNumber *)categoryIndex;
- (NSArray *)categoryRecordsForCategoryIndex:(NSNumber *)categoryIndex;
- (NSNumber *)categoryIdForCategoryIndex:(NSNumber *)categoryIndex editingContext:(FBEditingContext *)editingContext;

- (void)setBody:(Body *)body forBodyIndex:(NSNumber *)bodyIndex;
- (Body *)bodyForBodyIndex:(NSNumber *)bodyIndex;
- (NSDictionary *)bodyPropForBodyIndex:(NSNumber *)bodyIndex;
- (NSArray *)bodyRecordsForBodyIndex:(NSNumber *)bodyIndex;
- (NSNumber *)bodyIdForBodyIndex:(NSNumber *)bodyIndex editingContext:(FBEditingContext *)editingContext;

- (void)setProperty:(Property *)property forPropertyIndex:(NSNumber *)propertyIndex;
- (Property *)propertyForPropertyIndex:(NSNumber *)propertyIndex;
- (NSDictionary *)propertyPropForPropertyIndex:(NSNumber *)propertyIndex;
- (NSArray *)propertyRecordsForPropertyIndex:(NSNumber *)propertyIndex;
- (NSNumber *)propertyIdForPropertyIndex:(NSNumber *)propertyIndex editingContext:(FBEditingContext *)editingContext;

- (void)setLink:(Link *)link forLinkIndex:(NSNumber *)linkIndex;
- (Link *)linkForLinkIndex:(NSNumber *)linkIndex;
- (NSDictionary *)linkPropForLinkIndex:(NSNumber *)linkIndex;
- (NSArray *)linkRecordsForLinkIndex:(NSNumber *)linkIndex;
- (NSNumber *)linkIdForLinkIndex:(NSNumber *)linkIndex editingContext:(FBEditingContext *)editingContext;

- (void)setBibliography:(Bibliography *)bibliography forBibliographyIndex:(NSNumber *)bibliographyIndex;
- (Bibliography *)bibliographyForBibliographyIndex:(NSNumber *)bibliographyIndex;
- (NSDictionary *)bibliographyPropForBibliographyIndex:(NSNumber *)bibliographyIndex;
- (NSArray *)bibliographyRecordsForBibliographyIndex:(NSNumber *)bibliographyIndex;
- (NSNumber *)bibliographyIdForBibliographyIndex:(NSNumber *)bibliographyIndex editingContext:(FBEditingContext *)editingContext;

@end


@interface NSAttributedString (ObjectArchivingExtension)

- (NSString *)archivedTextWithCoder:(ObjectArchiver *)coder;
- (NSString *)archivedTextWithCoder:(ObjectArchiver *)coder properties:(NSMutableArray *)properties links:(NSMutableArray *)links;
- (void)encodeRange:(NSRange)range withCoder:(ObjectArchiver *)archiver;

@end


@interface Book (ObjectArchivingExtension)

+ (id)bookForBookIndex:(NSNumber *)bookIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;
- (void)encodeRootWithCoder:(ObjectArchiver *)archiver;
- (void)encodeWithCoder:(ObjectArchiver *)archiver;

@end


@interface Folder (ObjectArchivingExtension)

+ (id)folderForFolderIndex:(NSNumber *)folderIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;
- (void)encodeRootWithCoder:(ObjectArchiver *)archiver;
- (void)encodeWithCoder:(ObjectArchiver *)archiver;

@end


@interface Chapter (ObjectArchivingExtension)

+ (id)chapterForChapterIndex:(NSNumber *)chapterIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;
- (void)encodeWithCoder:(ObjectArchiver *)archiver;

@end

@interface FolderFile (ObjectArchivingExtension)

+ (id)fileForFileIndex:(NSNumber *)fileIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;
- (void)encodeWithCoder:(ObjectArchiver *)archiver;

@end


@interface Section (ObjectArchivingExtension)

+ (BOOL)shouldCreateNewPropertyForPropertyIndex:(NSNumber *)propertyIndex sectionIndex:(NSNumber *)sectionIndex coder:(ObjectUnarchiver *)coder;
+ (BOOL)shouldCreateNewLinkForLinkIndex:(NSNumber *)linkIndex sectionIndex:(NSNumber *)sectionIndex coder:(ObjectUnarchiver *)coder;
+ (BOOL)shouldCreateNewBodyForBodyIndex:(NSNumber *)bodIndex sectionIndex:(NSNumber *)sectionIndex coder:(ObjectUnarchiver *)coder;

+ (id)sectionForSectionIndex:(NSNumber *)sectionIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;
- (void)encodeRootWithCoder:(ObjectArchiver *)archiver;
- (void)encodeWithCoder:(ObjectArchiver *)archiver;
- (void)encodeWithCoder:(ObjectArchiver *)archiver alone:(BOOL)flag;
- (NSArray *)encodeWithCoder:(ObjectArchiver *)archiver sections:(NSArray *)sections;

@end


@interface Category (ObjectArchivingExtension)

+ (id)categoryForCategoryIndex:(NSNumber *)categoryIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;
- (void)encodeWithCoder:(ObjectArchiver *)archiver;

@end

@interface Document (ObjectArchivingExtension)

- (void)encodeWithCoder:(ObjectArchiver *)archiver;

@end


@interface Body (ObjectArchivingExtension)

+ (BOOL)shouldCreateNewPropertyForPropertyIndex:(NSNumber *)propertyIndex bodyIndex:(NSNumber *)bodyIndex coder:(ObjectUnarchiver *)coder;
+ (BOOL)shouldCreateNewLinkForLinkIndex:(NSNumber *)linkIndex bodyIndex:(NSNumber *)bodyIndex coder:(ObjectUnarchiver *)coder;
+ (BOOL)shouldCreateNewBodyForBodyIndex:(NSNumber *)bodIndex bodyIndex:(NSNumber *)bodyIndex coder:(ObjectUnarchiver *)coder;

+ (id)bodyForBodyIndex:(NSNumber *)bodyIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;
- (void)encodeWithCoder:(ObjectArchiver *)archiver;

@end


/*@interface Attachment (ObjectArchivingExtension)

+ (id)attachmentForAttachmentIndex:(NSNumber *)attachmentIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;
- (void)populatePropDic:(NSDictionary *)dic coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;
- (void)encodeRootWithCoder:(ObjectArchiver *)archiver;
- (void)encodeWithCoder:(ObjectArchiver *)archiver;

@end*/


@interface Property (ObjectArchivingExtension)

+ (BOOL)shouldCreateNewPropertyForPropertyIndex:(NSNumber *)propertyIndex propertyIndex:(NSNumber *)propIndex coder:(ObjectUnarchiver *)coder;
+ (BOOL)shouldCreateNewLinkForLinkIndex:(NSNumber *)linkIndex propertyIndex:(NSNumber *)propIndex coder:(ObjectUnarchiver *)coder;
+ (BOOL)shouldCreateNewBodyForBodyIndex:(NSNumber *)bodyIndex propertyIndex:(NSNumber *)propIndex coder:(ObjectUnarchiver *)coder;
+ (BOOL)shouldCreateNewSectionForSectionIndex:(NSNumber *)sectionIndex propertyIndex:(NSNumber *)propIndex coder:(ObjectUnarchiver *)coder;

+ (id)propertyForPropertyIndex:(NSNumber *)propertyIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;
- (void)populatePropDic:(NSDictionary *)dic coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;
- (void)encodeWithCoder:(ObjectArchiver *)archiver;

@end

/*@interface Theorem (ObjectArchivingExtension)

+ (id)theoremForTheoremIndex:(NSNumber *)theoremIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;
- (void)populatePropDic:(NSDictionary *)dic;
- (void)encodeRootWithCoder:(ObjectArchiver *)archiver;
- (void)encodeWithCoder:(ObjectArchiver *)archiver;

@end*/

/*@interface Enumerate (ObjectArchivingExtension)

+ (id)enumerateForEnumerateIndex:(NSNumber *)enumerateIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;
- (void)populatePropDic:(NSDictionary *)dic;
- (void)encodeWithCoder:(ObjectArchiver *)archiver;

@end*/


@interface Bibliography (ObjectArchivingExtension)

+ (id)bibliographyForBibliographyIndex:(NSNumber *)bibliographyIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;
- (void)populatePropDic:(NSDictionary *)dic;
- (void)encodeRootWithCoder:(ObjectArchiver *)archiver;
- (void)encodeWithCoder:(ObjectArchiver *)archiver;

@end


@interface Link (ObjectArchivingExtension)

+ (id)linkForLinkIndex:(NSNumber *)linkIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;
- (void)populatePropDic:(NSDictionary *)dic coder:(ObjectUnarchiver *)coder;
- (void)encodeWithCoder:(ObjectArchiver *)archiver;

@end


@interface NSMutableDictionary (NSMutableDictionaryObjectArchiverExtension)

- (NSNumber *)indexForKey:(NSNumber *)key;
- (NSArray *)valuesForKeys:(NSArray *)keys;

@end

@interface Record (ObjectArchivingExtension)

- (void)populatePropDic:(NSDictionary *)dic;

+ (NSArray *)recordsForPropertyIndex:(NSNumber *)propertyIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;

+ (NSArray *)recordsForLinkIndex:(NSNumber *)linkIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;

+ (NSArray *)recordsForBodyIndex:(NSNumber *)bodyIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;

+ (NSArray *)recordsForSectionIndex:(NSNumber *)sectionIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;

+ (NSArray *)recordsForBibliographyIndex:(NSNumber *)bibliographyIndex coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;

+ (NSArray *)recordsForRecodArray:(NSArray *)records coder:(ObjectUnarchiver *)coder editingContext:(FBEditingContext *)editingContext;

@end

