/* PropertyCarrierProtocol.h created by ahoesch on Thu 20-Dec-2001 */

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import "LaTeXGeneratorProtocol.h"

@class Body;
@class Property;
@class CountCenter;
@class ObjectUnarchiver;

@protocol PropertyCarrierProtocol

+ (NSString *)propertyTag;

// This method is used when the user creates a property
- (id)initWithBody:(Body *)body;

// This method is used in wakeUpPropertyCarriersOfClass:
- (id)initWithBody:(Body *)body attributes:(NSDictionary *)attributes unarchiver:(ObjectUnarchiver *)unarchiver;

// This method is supposed to replace the given range with a live representation of the property. The starting and stopping tag has already been removed when this method is called. This method is used to generate an attributed string from the database representation.
- (BOOL)awakeInRange:(NSRange)range attributedString:(NSMutableAttributedString *)attrString textView:(NSTextView *)textView length:(int *)newLength;

// This method is supposed to replace the given range with an XML representation. This method is used by preparePropertiesForSave
- (void)shutdownInRange:(NSRange)range attributedString:(NSMutableAttributedString *)attrString length:(int *)newLength;
- (void)shutdownInRange:(NSRange)range attributedString:(NSMutableAttributedString *)attrString length:(int *)newLength primaryKey:(NSNumber *)primaryKey;

// This method is supposed to remove its characters and attributes from the attributed string. This method is used by removeProperty
- (BOOL)removeInRange:(NSRange)range attributedString:(NSMutableAttributedString *)attrString textView:(NSTextView *)textView;

- (Property *)property;

- (NSString *)replacementForAttributedString:(NSAttributedString *)attrString latexGenerator:(id < LaTeXGeneratorProtocol >)generator;

@end
