/* SAFoundationExtensions.h created by ahoesch on Thu 14-Jun-2001 */
#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import <FBEnterprise/FBEnterprise.h>
#import "HTMLGeneratorProtocol.h"
#import "LaTeXGeneratorProtocol.h"
#import "PublicDefinitions.h"

@class TheoremCarrier;
@class EmphasisCarrier;
@class FormulaCarrier;
@class EnumerateCarrier;
@class WrapperCarrier;
@class FigureCarrier;

@class Body;
@class Section;
@class Property;
@class Link;
@class Enumerate;
@class CountCenter;

FOUNDATION_STATIC_INLINE BOOL SARangeInRange(NSRange range1, NSRange range2) {
    return (range1.location >= range2.location && NSMaxRange(range1) <= NSMaxRange(range2));
}

@interface NSAttributedString (NSAttributedStringSAFoundationExtension)

- (NSString *)extensionForFileType:(NSBitmapImageFileType)fileType;

- (unsigned int)paragraphStartForIndex:(unsigned int)index;
- (unsigned int)paragraphStopForIndex:(unsigned int)index;
- (Section *)anchorSection;

/* *******************************************************
   Attribute generation
   ******************************************************* */
+ (float)fontSizeForTitleAtLevel:(unsigned int)level;
+ (NSDictionary *)attributesForTitle:(Body *)title atLevel:(unsigned int)level;
+ (NSDictionary *)attributesForText;

- (BOOL)checkAttributeToLeftForAttributeName:(NSString *)attribteName;

/* *******************************************************
   Finding attributes
   ******************************************************* */
- (id)attributesAtIndex:(unsigned int)location;
- (id)attribute:(NSString *)attrName atIndex:(unsigned int)location;
- (id)attribute:(NSString *)attrName atIndex:(unsigned int)location longestEffectiveRange:(NSRangePointer)range;

- (NSRange)rangeOfTextAttachment:(NSTextAttachment *)textAttachment;

- (NSTextAttachment *)textAttachementAtIndex:(unsigned int)location;

- (EmphasisCarrier *)emphasisCarrierAtIndex:(unsigned int)location;
- (EmphasisCarrier *)emphasisCarrierAtIndex:(unsigned int)location longestEffectiveRange:(NSRangePointer)range;

- (TheoremCarrier *)theoremCarrierAtIndex:(unsigned int)location;
- (TheoremCarrier *)theoremCarrierAtIndex:(unsigned int)location longestEffectiveRange:(NSRangePointer)range;

- (FormulaCarrier *)formulaCarrierAtIndex:(unsigned int)location;
- (FormulaCarrier *)formulaCarrierAtIndex:(unsigned int)location longestEffectiveRange:(NSRangePointer)range;

- (FigureCarrier *)figureCarrierAtIndex:(unsigned int)location;
- (FigureCarrier *)figureCarrierAtIndex:(unsigned int)location longestEffectiveRange:(NSRangePointer)range;

- (WrapperCarrier *)wrapperCarrierAtIndex:(unsigned int)location;
- (WrapperCarrier *)wrapperCarrierAtIndex:(unsigned int)location longestEffectiveRange:(NSRangePointer)range;

- (EnumerateCarrier *)enumerateCarrierAtIndex:(unsigned int)location;
- (EnumerateCarrier *)enumerateCarrierAtIndex:(unsigned int)location longestEffectiveRange:(NSRangePointer)range;


- (Link *)linkAtIndex:(unsigned int)location;
- (Link *)linkAtIndex:(unsigned int)location longestEffectiveRange:(NSRangePointer)range;

- (Body *)bodyAtIndex:(unsigned int)location;
- (Body *)bodyAtIndex:(unsigned int)location longestEffectiveRange:(NSRangePointer)range;

- (Section *)sectionAtIndex:(unsigned int)location;
- (Section *)sectionAtIndex:(unsigned int)location longestEffectiveRange:(NSRangePointer)range;
- (Section *)sectionAtIndex:(unsigned int)location longestEffectiveRange:(NSRangePointer)range inRange:(NSRange)limit;

- (BOOL)sectionTitleAtIndex:(unsigned int)location;
- (BOOL)sectionTitleAtIndex:(unsigned int)location longestEffectiveRange:(NSRangePointer)range;
- (BOOL)sectionTitleAtIndex:(unsigned int)location longestEffectiveRange:(NSRangePointer)range inRange:(NSRange)limit;

- (BOOL)lastCharacterInRange:(NSRange)range attributeName:(NSString *)attributeName attribute:(id *)pattribute;

- (NSString *)latexRepresentationWithLaTeXGenerator:(id < LaTeXGeneratorProtocol >)generator;
- (NSString *)htmlRepresentationWithHTMLGenerator:(id < HTMLGeneratorProtocol >)generator;
- (NSString *)latexSectionTagForLevel:(unsigned int)level;

- (BOOL)latexStructureAtIndex:(int)index;

@end

@interface NSMutableAttributedString (NSMutableAttributedStringSAFoundationExtension)

- (void)setString:(NSString *)string;
- (void)insertString:(NSString *)string atIndex:(unsigned)index;
- (NSAttributedString *)attributedSubstringToIndex:(unsigned)index;

- (id)replaceControlCharactersForLateX;
- (id)replaceControlCharactersForHTML;
- (void)highlightSectionTitles;
- (void)dehighlightSectionTitles;

- (void)fixPropertyAttributesInRange:(NSRange)range;
- (void)fixPropertyAttributesInRange:(NSRange)range countCenter:(CountCenter *)countCenter;

- (void)fixTextAttributesInRange:(NSRange)range;

- (void)fixLinefeedAttributesInRange:(NSRange)range;
- (void)fixFontAttributesInRange:(NSRange)range;
- (void)fixFormulaAttributesInRange:(NSRange)range;
- (void)fixFormulaStyleAttributesInRange:(NSRange)range;
//- (void)fixParagraphStyleAttributesInRange:(NSRange)range;
- (NSRange)fixFormulaAlignmentsAtIndex:(int)aIndex constructive:(BOOL)flag;
- (NSRange)fixFormulaAlignmentsInRange:(NSRange)aRange constructive:(BOOL)flag;
- (void)fixFormulaLabelsInRange:(NSRange)aRange editedRange:(NSRange *)pRange;


- (SALineType)lineTypeAtIndex:(int)aIndex;

//- (NSDictionary *)fontAttributesForAttributeNames:(NSArray *)attributeNames atIndex:(int)index;
- (NSDictionary *)fontAttributesForAttributes:(NSDictionary *)attributeDic atIndex:(int)index;

- (void)setFontAttributes:(NSDictionary *)dic range:(NSRange)range;

@end

@interface NSString (NSStringSAFoundationExtension)

- (NSString *)stringAfterReplacingRef:(NSString *)aBezug;

@end


